PRAGMA foreign_keys = ON;

DROP TABLE IF EXISTS linkArchive; 
DROP TABLE IF EXISTS link; 
DROP TABLE IF EXISTS anchor;
DROP TABLE IF EXISTS marker; 
DROP TABLE IF EXISTS sketch;

CREATE TABLE sketch(
    artifactID TEXT PRIMARY KEY NOT NULL,
    timestamp TEXT NOT NULL,
    authors TEXT,
    annotation TEXT,
    title TEXT
);

CREATE TABLE marker(
    artifactID TEXT PRIMARY KEY NOT NULL,
    parentID TEXT NOT NULL,    
    timestamp TEXT,    
    annotation TEXT,    
    FOREIGN KEY(parentID) REFERENCES sketch(artifactID)
);

CREATE TABLE anchor(
    artifactID TEXT PRIMARY KEY NOT NULL,
    parentID TEXT,    
    timestamp TEXT,
    path TEXT,
    targetType TEXT,
    targetName TEXT,    
    projectName TEXT
);

CREATE TABLE link(
    id INTEGER PRIMARY KEY,
    artifactID1 TEXT,
    artifactID2 TEXT,
    timestamp TEXT,
    deprecated INT
);

CREATE TABLE linkArchive(
    id INTEGER PRIMARY KEY,
    artifactID1 TEXT,
    artifactID2 TEXT,
    timestamp TEXT,
    deprecated INT
);

INSERT INTO sketch VALUES ("s-b6787af4-9508-4925-962a-cdb3457f0192", "2013-12-02 18:12:11", "diehl", "This sketch visualizes the high-level structure of the SDRaytracer.", "SDRaytracer Overview");
INSERT INTO sketch VALUES ("s-2c465944-4116-4654-939f-0434229a6268", "2014-01-05 13:08:21", "diehl", "Lorem ipsum", "Pixel Raster");

INSERT INTO marker VALUES ("m-89525190-2571-402f-924f-974313767278", "s-b6787af4-9508-4925-962a-cdb3457f0192", "2013-12-04 14:11:13", "renderImage() calls raytrace() for each pixel on the display raster.");
INSERT INTO marker VALUES ("m-6b17cc93-704a-4a6b-ba08-d98da2374cad", "s-b6787af4-9508-4925-962a-cdb3457f0192", "2013-12-04 14:11:13", "raytrace() is called for each pixel.");
INSERT INTO marker VALUES ("m-f62b2fcf-ed69-4579-8b59-4df2e56dd43d", "s-b6787af4-9508-4925-962a-cdb3457f0192", "2013-12-04 14:11:13", "hitObject()");
INSERT INTO marker VALUES ("m-f65034f8-e704-4bfd-85f9-011362b6e363", "s-b6787af4-9508-4925-962a-cdb3457f0192", "2013-12-04 14:11:13", "lighting()");
INSERT INTO marker VALUES ("m-5d47a74b-b5d4-4a19-bda1-3320b4498015", "s-b6787af4-9508-4925-962a-cdb3457f0192", "2013-12-04 14:11:13", "raytrace()");
INSERT INTO marker VALUES ("m-7acd983f-8c73-491d-9801-52f504eb3995", "s-b6787af4-9508-4925-962a-cdb3457f0192", "2013-12-04 14:11:13", "raytrace() again");
INSERT INTO marker VALUES ("m-6ccdfa3b-1751-41f4-b822-c56012167543", "s-b6787af4-9508-4925-962a-cdb3457f0192", "2013-12-04 14:11:13", "");

INSERT INTO marker VALUES ("m-17fce670-5d07-40a0-80fa-212d9219fa98", "s-2c465944-4116-4654-939f-0434229a6268", "2013-12-04 14:11:13", "renderImage() calls raytract() for each pixel on the display raster.");

--define anchor
INSERT INTO anchor VALUES ("c-e104828f-5787-46f8-968b-0e0034adbbeb", "c-4d66a16d-04c2-48e7-910d-4b7018deb00c", "2013-12-04 14:32:13", "/edu/cg/SDRaytracer.java", "Method", "renderImage", "SDRaytracer");
INSERT INTO anchor VALUES ("c-e1530d7b-df9d-48e1-9961-44f33937de1a", "c-4d66a16d-04c2-48e7-910d-4b7018deb00c", "2013-12-04 15:14:03", "/edu/cg/SDRaytracer.java", "Method", "rayTrace", "SDRaytracer");
INSERT INTO anchor VALUES ("c-ce8851bc-89f2-4434-96bd-218ef9840889", "c-4d66a16d-04c2-48e7-910d-4b7018deb00c", "2013-12-04 17:18:58", "/edu/cg/SDRaytracer.java", "Method", "lighting", "SDRaytracer");
INSERT INTO anchor VALUES ("c-11752725-6841-41e6-8a10-b1e7186d5312", "c-4d66a16d-04c2-48e7-910d-4b7018deb00c", "2013-12-05 08:49:09", "/edu/cg/SDRaytracer.java", "Method", "hitObject", "SDRaytracer");
INSERT INTO anchor VALUES ("c-4d66a16d-04c2-48e7-910d-4b7018deb00c", null, "2013-12-04 13:15:11", "/edu/cg/SDRaytracer.java", "Class", "SDRaytracer", "SDRaytracer");
--define links to sketches
INSERT INTO link VALUES (null, "s-b6787af4-9508-4925-962a-cdb3457f0192", "c-4d66a16d-04c2-48e7-910d-4b7018deb00c", "2013-12-04 14:11:13", 0);
INSERT INTO link VALUES (null, "s-2c465944-4116-4654-939f-0434229a6268", "c-e1530d7b-df9d-48e1-9961-44f33937de1a", "2013-12-04 14:11:13", 0);
--define links to overview sketch
INSERT INTO link VALUES (null, "m-89525190-2571-402f-924f-974313767278", "c-e104828f-5787-46f8-968b-0e0034adbbeb", "2013-12-04 14:11:13", 0);
INSERT INTO link VALUES (null, "m-6b17cc93-704a-4a6b-ba08-d98da2374cad", "c-e1530d7b-df9d-48e1-9961-44f33937de1a", "2013-12-04 14:11:13", 0);
INSERT INTO link VALUES (null, "m-5d47a74b-b5d4-4a19-bda1-3320b4498015", "c-e1530d7b-df9d-48e1-9961-44f33937de1a", "2013-12-04 14:11:13", 0);
INSERT INTO link VALUES (null, "m-7acd983f-8c73-491d-9801-52f504eb3995", "c-e1530d7b-df9d-48e1-9961-44f33937de1a", "2013-12-04 14:11:13", 0);
INSERT INTO link VALUES (null, "m-f62b2fcf-ed69-4579-8b59-4df2e56dd43d", "c-11752725-6841-41e6-8a10-b1e7186d5312", "2013-12-04 14:11:13", 0);
INSERT INTO link VALUES (null, "m-f65034f8-e704-4bfd-85f9-011362b6e363", "c-11752725-6841-41e6-8a10-b1e7186d5312", "2013-12-04 14:11:13", 0);
--define links to pixel raster sketch
INSERT INTO link VALUES (null, "m-17fce670-5d07-40a0-80fa-212d9219fa98", "c-e104828f-5787-46f8-968b-0e0034adbbeb", "2013-12-04 14:11:13", 0);
INSERT INTO link VALUES (null, "m-17fce670-5d07-40a0-80fa-212d9219fa98", "c-e1530d7b-df9d-48e1-9961-44f33937de1a", "2013-12-04 14:11:14", 0);
