/**
 * This file contains the logic of the MenuPopup component.
 *
 * @author baltes
 * @see MenuPopup.html
 *
 */
var MenuPopup = (function(NewSketchPopup, OpenSketchPopup) {

    var MenuPopup = function(args) {
        //region Private members
        var that,
            $page,
            svgCanvas,
            $popupDiv,
            isOpen,
            openAfterClose,
            x, y;

        $page = args.$page;
        svgCanvas = args.svgCanvas;
        isOpen = false;
        openAfterClose = "";
        //endregion

        //region Getter/setter for private members
        Object.defineProperties(this, {
            "isOpen": {
                get: function() {
                    return isOpen;
                }
            },
            "openAfterClose": {
                get: function() {
                    return openAfterClose;
                },
                set: function(newValue) {
                    openAfterClose = newValue;
                }
            }
        });
        //endregion

        //region Private methods
        function create(callback) {
            $.mobile.loading('show');

            $.get("com/MenuPopup/MenuPopup.html", function (data) {
                var $newPopupDiv = $page
                    .append($.parseHTML(data)[0])
                    .trigger('create');

                $popupDiv = $newPopupDiv
                    .find('#menuPopup')
                    .popup();

                /* jQuery mobile binds _eatEventAndClose to the popup screen to prevent clicking on it, revert that */
                /* (again, workaround for iOS, problems with data-dismissible=true) */
                $page
                    .find('#menuPopup-screen')
                    .unbind('vclick')
                    .on('vclick', function() {
                        if (isOpen) {
                            close();
                        }
                    });

                $popupDiv
                    .on("popupafteropen", function() {
                        util.log("open_menu");
                    })
                    .on('popupafterclose', function() {
                        util.log("close_menu");

                        isOpen = false;

                        switch (openAfterClose) {
                            case "newsketch":
                                popups.newSketchPopup.open();
                                break;
                            case "opensketch":
                                popups.openSketchPopup.open();
                                break;
                            default:
                            // do nothing
                        }

                        openAfterClose = "";
                    });

                /* new sketch popup */
                popups.newSketchPopup = new NewSketchPopup({
                    $page: $page,
                    svgCanvas: svgCanvas
                });
                popups.newSketchPopup.create();

                $popupDiv
                    .find('#anchorNewSketch')
                    .on('vclick', function() {
                        if (isOpen) {
                            popups.newSketchPopup.open();
                        }
                    });

                /* open sketch popup */
                popups.openSketchPopup = new OpenSketchPopup({
                    $page: $page,
                    svgCanvas: svgCanvas
                });
                popups.openSketchPopup.create();

                $popupDiv
                    .find('#anchorOpenSketch')
                    .on('vclick', function() {
                        if (isOpen) {
                            popups.openSketchPopup.open();
                        }
                    });

                if (callback) {
                    callback();
                }

                $.mobile.loading('hide');
            });
        }

        function open(args) {
            if (args && args.x && args.y) {
                x = args.x;
                y = args.y;
            }

            if (popups.panelPopup.isOpen) {
                popups.panelPopup.openAfterClose = "menu";
                popups.panelPopup.close();
            } else {
                if (util.isIOSDevice()) {
                    /* workaround for iOS, since Mobile Safari seems to trigger a vclick on the popup screen directly after opening it */
                    window.setTimeout(function() {
                        isOpen = true;
                    }, 800);
                } else {
                    isOpen = true;
                }

                /* positionTo: 'origin' did not work (popup was centered), therfore use provided coordinates */
                if (x && y) {
                    $popupDiv.popup('open', {
                        x: x,
                        y: y,
                        positionTo: 'origin'
                    });
                }
            }
        }

        function close() {
            $popupDiv.popup('close');
        }
        //endregion

        //region Privileged methods
        this.create = create;
        this.open = open;
        this.close = close;
        //endregion
    };

    return MenuPopup;

})(NewSketchPopup, OpenSketchPopup);