/**
 * This file contains the logic of the MetaInfoPopup component.
 * It is used to display and modify the metainformation about a whole sketch.
 *
 * @author baltes
 * @see MetaInfoPopup.html, MetaInfoPopup.css
 *
 */
var MetaInfoPopup = (function(util, icons) {

    // TODO : iOS bug: when one form element is focussed, all elements get a bold font.

    var MetaInfoPopup = function(args) {
        //region Private members
        var svgCanvas,
            $popupDiv,
            isOpen,
            openAfterClose,
            reopenElement,
            $mainGroup,
            $artifact_dom,
            $annotation_dom,
            $id_dom,
            $authors_dom,
            $title_dom,
            $timestamp_dom,
            $button_dom,
            artifactPath,
            artifactType,
            authors_old,
            annotation_old,
            title_old,
            artifact_id_old;

        icons.load();

        svgCanvas = args.svgCanvas;
        $popupDiv = null;
        isOpen = false;
        openAfterClose = "";
        reopenElement = null;
        //endregion

        //region Getter/setter for private members
        Object.defineProperties(this, {
            "isOpen" : {
                get: function() {
                    return isOpen;
                }
            },
            "openAfterClose": {
                get: function() {
                    return openAfterClose;
                },
                set: function(newValue) {
                    openAfterClose = newValue;
                }
            }
        });
        //endregion

        //region Private methods
        function create(callback) {
            $.mobile.loading('show');

            /* load css for this component */
            util.loadCSS("/com/MetaInfoPopup/MetaInfoPopup.css", function() {
                /* load html for this component */
                $.get("/com/MetaInfoPopup/MetaInfoPopup.html", function (data) {
                    var $newPopupDiv = svgCanvas.$page
                        .append($.parseHTML(data)[0])
                        .trigger('create');

                    $popupDiv = $newPopupDiv
                        .find('#metaInfoPopup')
                        .popup();

                    $id_dom = $popupDiv.find('#metaInfoID');
                    $authors_dom = $popupDiv.find('#metaInfoAuthors');
                    $timestamp_dom = $popupDiv.find('#metaInfoTimestamp');
                    $annotation_dom = $popupDiv.find('#metaInfoAnnotation');
                    $artifact_dom = $popupDiv.find('#metaInfoArtifact');
                    $button_dom = $popupDiv.find('#metaInfoFollowLink');
                    $title_dom = $popupDiv.find('#metaInfoTitle');

                    util.disableTextSelection($popupDiv.find('div.header'));
                    util.disableTextSelection($popupDiv.find('div.sl-header-title'));
                    util.disableTextSelection($artifact_dom);

//                    /* logging */
//                    $authors_dom.on("change", function() {
//                        util.log("authors_edited;" + $authors_dom.val());
//                    });
//                    $date_dom.on("change", function() {
//                        util.log("date_edited;" + $date_dom.val());
//                    });
//                    $time_dom.on("change", function() {
//                        util.log("time_edited;" + $time_dom.val());
//                    });

                    /* register event handlers */
                    $popupDiv
                        .find('#metaInfoButtonSave')
                        .on('vclick', function() {
                            saveAndClose();
                            //util.log("close_and_safe_metainfo");
                        });
                    $popupDiv
                        .find('#metaInfoButtonClose')
                        .on('vclick', function() {
                            close();
                            //util.log("close_metainfo");
                        });

                    $button_dom
                        .on("vclick", function() {
                            var artifact_id = $artifact_dom.val();
                            if (artifact_id) {
                                var showBroadcast = {
                                    "id": util.generateNewID(),
                                    "message": "show",
                                    "type": "broadcast",
                                    "data": {
                                        "artifactID": artifact_id,
                                        "clients": null
                                    }
                                };

                                window.websocket.send(JSON.stringify(showBroadcast));
                            }
                        });

                    $artifact_dom
                        .on("vclick", function() {
                            //openAfterClose = "navigation";
                            //close();
                        })
                        .on("blur", function() {
                            util.setTextInputCursorPos($(this)[0], 0);
                        });

                    $popupDiv
                        .on('popupafteropen', function() {
                            util.log("open_metainfo");

                            if (util.isIOSDevice()) {
                                util.unfocusInputs($popupDiv, $popupDiv.find('#metaInfoID'));
                            }
                        })
                        .on('popupafterclose', function() {
                            switch (openAfterClose) {
                                case "panel":
                                    popups.panelPopup.open({
                                        element: reopenElement
                                    });
                                    reopenElement = null;
                                    break;
                                case "navigation":
                                    popups.navigationPopup.open({
                                        element: svgCanvas.root
                                    });
                                    break;
                                default:
                                    // do nothing
                            }

                            $annotation_dom.val('');

                            if (util.isIOSDevice()) {
                                util.unfocusInputs($popupDiv, $popupDiv.find('#metaInfoID'));
                            }

                            openAfterClose = "";
                            isOpen = false;
                        });

                    $(window).on('orientationchange', function() {
                        if (util.isIOSDevice()) {
                            util.unfocusInputs($popupDiv, $popupDiv.find('#metaInfoID'));
                        }
                    });

                    $(window).on('resize', function() {
                        reposition();
                    });

                    /* call callback of create function */
                    if (callback) {
                        callback();
                    }

                    $.mobile.loading('hide');
                });
            });
        }

        function open(args) {
            var log;

            $id_dom.val('');
            $authors_dom.val('');
            $timestamp_dom.val('');
            $annotation_dom.val('');
            $artifact_dom.val('');

            authors_old = "";
            annotation_old = "";
            title_old = "";
            artifact_id_old = ""

            if (svgCanvas.$root) {

                var sketch_id = svgCanvas.$root.attr("id");
                // get sketch metadata
                $.get("/sketches/metadata/" + sketch_id, function (data) {
                    authors_old = data.authors;
                    annotation_old = util.decode(data.annotation);
                    title_old = data.title;

                    $id_dom.val(sketch_id);
                    $authors_dom.val(authors_old);
                    $timestamp_dom.val(data.timestamp);
                    $annotation_dom.val(annotation_old);
                    $title_dom.val(title_old);

                    if (window.changelog[sketch_id]) {
                        if (window.changelog[sketch_id]["authors"]) {
                            $authors_dom.val(window.changelog[sketch_id]["authors"]);
                        }
                        if (window.changelog[sketch_id]["annotation"]) {
                            $annotation_dom.val(window.changelog[sketch_id]["annotation"]);
                        }
                        if (window.changelog[sketch_id]["title"]) {
                            $title_dom.val(window.changelog[sketch_id]["title"]);
                        }
                    }
                });

                $.get("/sketches/links/" + sketch_id, function (data) {
                    if (data) {
                        artifact_id_old = data[0];
                        $artifact_dom.val(artifact_id_old);

                        if (window.code_id) {
                            $artifact_dom.val(window.code_id);

                            if (window.changelog[sketch_id]) {
                                window.changelog[sketch_id]["artifact-id"] = window.code_id;
                            } else {
                                window.changelog[sketch_id] = {
                                    "artifact-id": window.code_id
                                }
                            }
                        }
                    }
                });

                //util.updateArtifactTextInput(artifactPath, artifactType, $artifact_dom);

                if (popups.panelPopup.isOpen) {
                    popups.panelPopup.openAfterClose = "metainfo";
                    reopenElement = popups.panelPopup.element;
                    popups.panelPopup.close();
                    return;
                }

                $popupDiv.popup('open');
                isOpen = true;
            }
        }

        function close() {
            $popupDiv.popup('close');
        }

        function saveAndClose() {
            saveMetaInfo();
            close();
        }

        function reposition() {
            $popupDiv.popup("reposition", {positionTo: "window"} );
        }

        function saveMetaInfo() {
//            var oldAnnotation,
//                newAnnotation,
//                oldAuthors,
//                newAuthors,
//                oldDate,
//                newDate,
//                oldTime,
//                newTime;
//
//            if (svgCanvas.$root) {
//                $mainGroup = svgCanvas.$root.find('g:first');
//
//                oldAnnotation  = annotation_svg;
//                newAnnotation = $annotation_dom.val();
//
//                if (newAnnotation && (newAnnotation != oldAnnotation)) {
//                    $mainGroup.attr('data:annotation', util.encode(newAnnotation));
//                    annotation_svg = newAnnotation;
//
//                    util.log("annotation_edited;" +  util.encode(newAnnotation));
//                }
//
//                oldAuthors = authors_svg;
//                newAuthors = $authors_dom.val();
//
//                if (newAuthors && (newAuthors != oldAuthors)) {
//                    $mainGroup.attr('data:authors', newAuthors);
//                }
//
//                oldDate = date_svg;
//                newDate = $date_dom.val();
//
//                if (newDate && (newDate != oldDate)) {
//                    $mainGroup.attr('data:date', newDate);
//                }
//
//                oldTime = time_svg;
//                newTime = $time_dom.val();
//
//                if (newTime && (newTime != oldTime)) {
//                    $mainGroup.attr('data:time', newTime);
//                }
//            }

            var authors_new,
                annotation_new,
                title_new;

            if (!svgCanvas.$root) {
                return;
            }

            var sketch_id = svgCanvas.$root.attr("id");
            authors_new = $authors_dom.val();
            annotation_new = $annotation_dom.val();
            title_new = $title_dom.val();

            if (authors_new != authors_old) {
                if (!window.changelog[sketch_id]) {
                    window.changelog[sketch_id] = {
                        "authors": authors_new
                    }
                } else {
                    window.changelog[sketch_id]["authors"] = authors_new
                }
            }

            if (annotation_new != annotation_old) {
                if (!window.changelog[sketch_id]) {
                    window.changelog[sketch_id] = {
                        "annotation": annotation_new
                    }
                } else {
                    window.changelog[sketch_id]["annotation"] = annotation_new
                }
            }

            if (title_new != title_old) {
                if (!window.changelog[sketch_id]) {
                    window.changelog[sketch_id] = {
                        "title": title_new
                    }
                } else {
                    window.changelog[sketch_id]["title"] = title_new
                }
            }
        }
        //endregion

        //region Privileged methods
        this.create = create;
        this.open = open;
        this.close = close;
        //endregion
    };

    return MetaInfoPopup;

})(util, icons);