/**
 * This file contains the logic of the NavigationPopup component, which loads the NavigationView component into an iFrame.
 * It is used to browse through the structure of a project.
 *
 * @author baltes
 * @see NavigationPopup.html, NavigationPopup.css, NavigationView.html, NavigationView.css, NavigationView.js
 *
 */
var NavigationPopup = (function(util) {

    var NavigationPopup = function(args) {
        //region Constants
        Object.defineProperties(this, {
            "HEADER_HEIGHT": {
                value: 37
            }
        });
        //endregion

        //region Private members
        var that,
            $page,
            svgCanvas,
            $popupDiv,
            $iFrame,
            element,
            isOpen,
            openAfterClose,
            newSketchPopup,
            $view,
            $header,
            artifactPath,
            artifactType,
            clearArtifact;

        that = this;
        $page = args.$page;
        svgCanvas = args.svgCanvas;
        $popupDiv = null;
        element = null;
        isOpen = false;
        openAfterClose = "";
        artifactPath = "";
        artifactType = "";
        clearArtifact = false;
        //endregion

        //region Getter/setter for private members
        Object.defineProperties(this, {
            "isOpen" : {
                get: function() {
                    return isOpen;
                }
            },
            "openAfterClose": {
                get: function() {
                    return openAfterClose;
                },
                set: function(newValue) {
                    openAfterClose = newValue;
                }
            }
        });
        //endregion

        //region Private methods
        function create(callback) {
            $.mobile.loading('show');

            /* load css for this component */
            util.loadCSS("/com/NavigationPopup/NavigationPopup.css", function() {
                /* load html for this component */
                $.get("/com/NavigationPopup/NavigationPopup.html", function (data) {
                    var $newPopupDiv = $page
                        .append($.parseHTML(data)[0])
                        .trigger('create');

                    $popupDiv = $newPopupDiv
                        .find('#navigationPopup')
                        .popup();

                    util.disableTextSelection($popupDiv.find('div.header'));
                    util.disableTextSelection($popupDiv.find('div.sl-header-title'));

                    $iFrame = $popupDiv.find("#navigationIFrame");
                    $header = $popupDiv.find('#navigationHeaderTitle');

                    if ($iFrame.get(0).contentWindow) {
                        $view = $iFrame.get(0).contentWindow.navigationView;
                    }

                    /* register event handlers */
                    $popupDiv
                        .find('#navigationButtonClose')
                        .on('vclick', function() {
                            util.log("close_navigationview");
                            close();
                        });

                    $popupDiv
                        .find('#navigationButtonSave')
                        .on('vclick', function() {
                            util.log("close_and_save_navigationview");
                            saveAndClose();
                        });

                    $popupDiv
                        .find('#navigationButtonDelete')
                        .on('vclick', function() {
                            util.log("close_and_clear_navigationview");
                            clearArtifact = true;
                            close();
                        });

                    $popupDiv
                        .on('popupafteropen', function() {
                            util.log("open_navigationview");

                            isOpen = true;
                            $.mobile.loading('hide');
                        })
                        .on('popupbeforeposition', function() {
                            $popupDiv
                                .css( "width", 0.95*window.innerWidth + "px" )
                                .css( "height", 0.95*window.innerHeight + "px");

                            $iFrame
                                .css( "height", 0.95*window.innerHeight - that.HEADER_HEIGHT + "px");
                        })
                        .on('popupafterclose', function() {
                            switch (openAfterClose) {
                                case "panel":
                                    popups.panelPopup.open({
                                        element: element,
                                        artifactPath: artifactPath,
                                        artifactType: artifactType,
                                        clear: clearArtifact
                                    });
                                    break;
                                case "metainfo":
                                    popups.metaInfoPopup.open({
                                        artifactPath: artifactPath,
                                        artifactType: artifactType,
                                        clear: clearArtifact
                                    });
                                    break;
                                case "newsketch":
                                    if ($view) {
                                        popups.newSketchPopup.open({
                                            artifactPath: artifactPath,
                                            artifactType: artifactType,
                                            clear: clearArtifact
                                        });
                                    }
                                    break;
                                default:
                                // do nothing
                            }

                            openAfterClose = "";
                            isOpen = false;
                            element = null;
                        });

                    $(window).on("resize", function() {
                        reposition();
                    });

                    /* call callback of create function */
                    if (callback) {
                        callback();
                    }

                    $.mobile.loading('hide');
                });
            });
        }

        function open(args) {
            var $element,
                elementName,
                $mainGroup,
                artifactPath,
                artifactType;

            $.mobile.loading('show'); // hidden when event popupafteropen is triggered

            $view = $iFrame.get(0).contentWindow.navigationView;
            $view.clear();
            clearArtifact = false;

            if (args && args.element) {
                element = args.element;
                $element = $(args.element);
                $mainGroup = svgCanvas.$root.find("g:first");

                if ($element.is("svg")) {
                    elementName = $mainGroup.attr("id");
                    artifactPath = $mainGroup.attr("data:artifact-path");
                    artifactType = $mainGroup.attr("data:artifact-type");
                    openAfterClose = "metainfo"; // open metainfo popup after navigation popup
                } else {
                    elementName = $element.attr("id");
                    artifactPath = $element.attr("data:artifact-path");
                    artifactType = $element.attr("data:artifact-type");
                    if (!(artifactPath && artifactPath)) { // use artifact of sketch as default
                        artifactPath = $mainGroup.attr("data:artifact-path");
                        artifactType = $mainGroup.attr("data:artifact-type");
                    }
                    openAfterClose = "panel"; // open panel after navigation popup
                }

                $view.artifactPath = artifactPath;
                $view.artifactType = artifactType;

                setPopupTitle('Select linked artifact for: ', elementName);
            } else {
                setPopupTitle('Select linked artifact for new sketch:');
            }

            if (args && args.artifactPath && args.artifactType) {
                $view.artifactPath = args.artifactPath;
                $view.artifactType = args.artifactType;
            }

            $view.loadProjectIndex("/projects/", function() {
                if (popups.panelPopup.isOpen) {
                    popups.panelPopup.openAfterClose = "navigation";
                    popups.panelPopup.close();
                } else {
                    $popupDiv.popup('open');
                }
            });
        }

        function close() {
            $view = $iFrame.get(0).contentWindow.navigationView;

            artifactPath = "";
            artifactType = "";

            $view.clear();
            $popupDiv.popup('close');
        }

        function saveAndClose() {
            $view = $iFrame.get(0).contentWindow.navigationView;

            artifactPath = $view.artifactPath;
            artifactType = $view.artifactType;

            $view.clear();
            $popupDiv.popup('close');
        }

        function reposition() {
            $popupDiv.popup("reposition", {positionTo: "window"} );
        }

        function setPopupTitle(text, elementName) {
            var $newDiv;

            $header
                .empty()
                .append(text);

            if (elementName) {
                $newDiv = $('<div>')
                    .css('display', 'inline')
                    .css('padding-left', '0.5em')
                    .css('font-family', '"Courier New", monospace')
                    .text(elementName);

                $header
                    .append($newDiv);
            }
        }
        //endregion

        //region Privileged methods
        this.create = create;
        this.open = open;
        //endregion
    };

    return NavigationPopup;

})(util);