/**
 * This file contains the logic of the NewSketchPopup component.
 * It is used to create and upload new sketches.
 *
 * @author baltes
 * @see NewSketchPopup.html, NewSketchPopup.css
 *
 */
var NewSketchPopup = (function(util) {

    //TODO : add image preview right of metainfo when an image is selected

    var NewSketchPopup = function(args) {
        //region Private members
        var $page,
            svgCanvas,
            menuPopup,
            $popupDiv,
            isOpen,
            openAfterClose,
            $image_dom,
            $title_dom,
            $authors_dom,
            $annotation_dom,
            $artifact_dom,
            //artifactPath,
            //artifactType;

        $page = args.$page;
        svgCanvas = args.svgCanvas;
        isOpen = false;
        openAfterClose = "";
        //artifactPath = "";
        //artifactType = "";
        //endregion

        //region Getter/setter for private members
        Object.defineProperties(this, {
            "isOpen" : {
                get: function() {
                    return isOpen;
                }
            }
        });
        //endregion

        //region Private methods
        function create(callback) {
            $.mobile.loading('show');

            /* load css for this component */
            util.loadCSS("/com/NewSketchPopup/NewSketchPopup.css", function() {
                /* load html for this component */
                $.get("/com/NewSketchPopup/NewSketchPopup.html", function (data) {
                    var $newPopupDiv;

                    $newPopupDiv = $page
                        .append($.parseHTML(data)[0])
                        .trigger('create');

                    $popupDiv = $newPopupDiv
                        .find('#newSketchPopup')
                        .popup();

                    $image_dom = $popupDiv.find('#newSketchImage');
                    $title_dom = $popupDiv.find('#newSketchTitle');
                    $authors_dom = $popupDiv.find('#newSketchAuthors');
                    $annotation_dom = $popupDiv.find('#newSketchAnnotation');
                    $artifact_dom = $popupDiv.find('#newSketchArtifact');

                    util.disableTextSelection($popupDiv.find('div.header'));
                    util.disableTextSelection($popupDiv.find('div.sl-header-title'));
                    //util.disableTextSelection($artifact_dom);

/*                    *//* logging *//*
                    $image_dom.on("change", function() {
                        util.log("image_selected;" + $image_dom.val());
                    });
                    $authors_dom.on("change", function() {
                        util.log("authors_edited;" + $authors_dom.val());
                    });
                    $annotation_dom.on("change", function() {
                        util.log("annotation_edited;" + $annotation_dom.val());
                    });*/

                    /* register event handlers */
                    $popupDiv
                        .find('#newSketchButtonClose')
                        .on('vclick', function() {
                            close();
                        });

                    $popupDiv
                        .find('#newSketchButtonUpload')
                        .on('vclick', function(event) {
                            /* prevent closing of popup */
                            event.preventDefault();
                            upload();
                        });

                    /*$artifact_dom
                        .on("vclick", function() {
                            openAfterClose = 'newsketch';
                            close();
                        })
                        .on("blur", function() {
                            util.setTextInputCursorPos($(this)[0], 0);
                        });*/

                    $popupDiv
                        .on('popupafteropen', function() {
                            if (util.isIOSDevice()) {
                                util.unfocusInputs($popupDiv, $artifact_dom); // important to unfocus here and not in open function
                            }
                            isOpen = true;
                            util.log("open_newsketch_popup");
                        })
                        .on('popupafterclose', function() {
                            util.log("close_newsketch_popup");

                            switch (openAfterClose) {
                                case 'newsketch':
                                    /*popups.navigationPopup.openAfterClose = 'newsketch';
                                    popups.navigationPopup.open({
                                        artifactPath: artifactPath,
                                        artifactType: artifactType
                                    });*/
                                    break;
                                default:
                                // do nothing
                            }

                            if (util.isIOSDevice()) {
                                util.unfocusInputs($popupDiv, $authors_dom);
                            }
                            openAfterClose = "";
                            isOpen = false;
                        });

                    $(window).on('orientationchange', function() {
                        if (util.isIOSDevice()) {
                            util.unfocusInputs($popupDiv, $authors_dom);
                        }
                    });

                    $(window).on('resize', function() {
                        reposition();
                    });

                    /* call callback of create function */
                    if (callback) {
                        callback();
                    }

                    $.mobile.loading('hide');
                });
            });
        }

        function open(args) {
            var path,
                selectedNode;

/*            if (args) {
                if (args.clear) {
                    artifactPath = "";
                    artifactType = "";
                    util.updateArtifactTextInput(artifactPath, artifactType, $artifact_dom);
                    util.log("artifact_cleared");
                } else if (args.artifactPath && args.artifactType) {
                    artifactPath = args.artifactPath;
                    artifactType = args.artifactType;
                    util.updateArtifactTextInput(artifactPath, artifactType, $artifact_dom);
                    util.log("artifact_changed;" + $artifact_dom.val());
                }
            }*/

            if (popups.menuPopup.isOpen) {
                popups.menuPopup.openAfterClose = "newsketch";
                popups.menuPopup.close();
            } else {
                $popupDiv.popup('open');
            }

            if (window.code_id) {
                $artifact_dom.val(window.code_id);
            }

        }

        function close() {
            $popupDiv.popup('close');
        }

        function reposition() {
            $popupDiv.popup("reposition", {positionTo: "window"} );
        }

        function clear() {
//            util.disableLogging();

            $image_dom.val("");
            $title_dom.val("");
            $authors_dom.val("");
            $annotation_dom.val("");

//            artifactPath = "";
//            artifactType = "";
            // util.updateArtifactTextInput(artifactPath, artifactType, $artifact_dom);

//            util.enableLogging();
        }

        function upload() {
            var data,
                imageFile,
                authors,
                annotation,
                title,
                artifact_id,
                svgUrl;

            imageFile = $image_dom.get(0).files[0];

            if (!imageFile || imageFile.type != 'image/jpeg') { // no image selected
                $image_dom.val("");
                alert("Please select a jpeg file.");
                return;
            }

            $.mobile.loading('show');

            title = $title_dom.val();
            authors = $authors_dom.val();
            annotation = $annotation_dom.val();
            artifact_id = $artifact_dom.val();

            data = new FormData;
            data.append("title", title);
            data.append("authors", authors);
            data.append("annotation", util.encode(annotation));
            data.append("artifact-id", artifact_id);
            data.append("image", imageFile);

            $.ajax({
                type: "POST",
                url: "/sketches/",
                data: data,
                async: false,
                mimeType:"multipart/form-data",
                contentType: false,
                cache: false,
                processData: false,
                success: function(id) {
                    svgUrl = "/sketches/svg/" + id;
                    svgCanvas.initialize(svgUrl);
                    util.log("upload_new_sketch;" + svgUrl);
                    clear();
                    $.mobile.loading('hide');
                    close();
                },
                error: function(jqXHR) {
                    alert("An ajax error occured.\n\n"
                        + "Status: " + jqXHR.status + "\n"
                        + "Status text: " + jqXHR.statusText);
                    $.mobile.loading('hide');
                }
            });
           // };

            //fileReader.readAsBinaryString(imageFile); // triggers load event on file reader
        }
        //endregion

        //region Privileged methods
        this.create = create;
        this.open = open;
        this.close = close;
        //endregion
    };

    return NewSketchPopup;

})(util);
