/**
 * This file contains the logic of the OpenSketchPopup component, which loads the OpenSketchView component into an iFrame.
 * It is used to open exisiting sketches.
 *
 * @author baltes
 * @see OpenSketchPopup.html, OpenSketchPopup.css, OpenSketchView.html, OpenSketchView.css, OpenSketchView.js
 *
 */
var OpenSketchPopup = (function(util) {

    var OpenSketchPopup = function(args) {
        //region Constants
        Object.defineProperties(this, {
            "MARGIN_TOP": {
                value: 37
            }
        });
        //endregion

        //region Private members
        var that,
            $page,
            svgCanvas,
            $popupDiv,
            $iFrame,
            isOpen;

        that = this;
        $page = args.$page;
        svgCanvas = args.svgCanvas;
        isOpen = false;
        //endregion

        //region Getter/setter for private members
        Object.defineProperties(this, {
            "isOpen" : {
                get: function() {
                    return isOpen;
                }
            }
        });
        //endregion

        //region Private methods
        function create(callback) {
            $.mobile.loading('show');

            /* load css for this component */
            util.loadCSS("/com/OpenSketchPopup/OpenSketchPopup.css", function() {
                /* load html for this component */
                $.get("/com/OpenSketchPopup/OpenSketchPopup.html", function (data) {
                    var $newPopupDiv = $page
                        .append($.parseHTML(data)[0])
                        .trigger('create');

                    $popupDiv = $newPopupDiv
                        .find('#openSketchPopup')
                        .popup();

                    util.disableTextSelection($popupDiv.find('div.header'));
                    util.disableTextSelection($popupDiv.find('div.sl-header-title'));

                    $iFrame = $popupDiv.find("#openSketchIFrame");

                    /* register event handlers */
                    $popupDiv
                        .find('#openSketchButtonClose')
                        .on('vclick', function() {
                            close();
                        });

                    $popupDiv
                        .on('popupbeforeposition', function() {
                            resize();
                        })
                        .on('popupafteropen', function() {
                            //util.log("open_opensketch_popup");
                            isOpen = true;
                        })
                        .on('popupafterclose', function() {
                            //util.log("close_opensketch_popup");
                            isOpen = false;
                        });

                    $(window).on('resize', function() {
                        resize();
                        reposition();
                    });

                    /* call callback of create function */
                    if (callback) {
                        callback();
                    }

                    $.mobile.loading('hide');
                });
            });
        }

        function open() {
            var $view;

            if (!$iFrame) {
                return;
            }

            $view = $iFrame.get(0).contentWindow.openSketchView;

            if (!$view) {
                return;
            }

            $.mobile.loading("show");
            $view.resize();
            $view.loadSketches("/sketches/", function() {
                /* set properties for openSketchView */
                $view.svgCanvas = svgCanvas;
                $view.openSketchPopup = that;

                $.mobile.loading("hide");

                if (popups.menuPopup.isOpen) {
                    popups.menuPopup.openAfterClose = "opensketch";
                    popups.menuPopup.close();
                } else {
                    $popupDiv.popup('open');
                }
            });
        }

        function close() {
            $popupDiv.popup('close');
        }

        function reposition() {
            $popupDiv.popup("reposition", {positionTo: "window"} );
        }

        function resize() {
            $popupDiv
                .css( "width", 0.6*window.innerWidth + "px" )
                .css( "height", 0.95*window.innerHeight + "px");

            $iFrame
                .css( "height", $popupDiv.innerHeight() - that.MARGIN_TOP + "px");
        }
        //endregion

        //region Privileged methods
        this.create = create;
        this.open = open;
        this.close = close;
        //endregion
    };

    return OpenSketchPopup;

})(util);