/**
 * This file contains the logic of the OpenSketchView component.
 * It is used to open existing sketches.
 *
 * @author baltes
 * @see OpenSketchView.html, OpenSketchView.css
 *
 */
var OpenSketchView = (function(util) {

    var OpenSketchView = function() {
        //region Constants
        Object.defineProperties(this, {
            "BOTTOM_MARGIN": {
                value: 20
            }
        });
        //endregion

        //region Private members
        var that,
            availableHeight,
            $imageList,
            svgCanvas,
            openSketchPopup,
            index;

        that = this;
        index = 1;
        initialize();
        //endregion

        //region Getter/setter for private members
        Object.defineProperties(this, {
            "svgCanvas": {
                get: function() {
                    return svgCanvas;
                },
                set: function(newValue) {
                    svgCanvas = newValue;
                }
            },
            "openSketchPopup": {
                get: function() {
                    return openSketchPopup;
                },
                set: function(newValue) {
                    openSketchPopup = newValue;
                }
            }
        });
        //endregion

        //region Private methods
        function initialize() {
            resize();

            $(window).on("resize", function() {
                resize();
            });

            $imageList = $(".image-list");

            $imageList
                .on('vclick', 'img', function(event) {
                    var $img = $(event.target);

                    //util.log("open_sketch;" + $img.attr('data-svgpath') + ";" + $img.attr('data-index') + "/" + (index-1));

                    if (svgCanvas && openSketchPopup) {
                        svgCanvas.initialize($img.attr('data-svgpath')); // cp. http://www.w3.org/html/wg/drafts/html/master/dom.html#embedding-custom-non-visible-data-with-the-data-*-attributes
                        openSketchPopup.close();
                    }
                });
        }

        function resize() {
            availableHeight = window.innerHeight-that.BOTTOM_MARGIN; // keep a small margin to the bottom

            /* jQuery mobile seems to set the min height on opening of popup, therefore reset it */
            $('#openSketchPage')
                .css("min-height", availableHeight)
                .css("height", availableHeight);

            $(".image-list")
                .scrollLeft(0)
                .scrollTop(0)
                .css("height", availableHeight + "px");

        }

        function loadSketches(path, callback) {
            index = 1;

            $.get(path, function(jsonArray) {
                $imageList
                    .find("img")
                    .remove();

                loadImage(jsonArray, 0, callback);
            }, "json");

            /* remove bottom margin for last image */
            $imageList
                .find("img:last")
                .css("margin-bottom", "0");
        }

        function loadImage(jsonArray, pos, callback) {
            var newImg;

            if (pos >= jsonArray.length) {

                if (pos === 0 ) { // call callback also if no image has been added
                    callback();
                }

                return;
            }

            newImg = $('<img>')
                .attr('data-index', index)
                .attr('data-svgpath', "/sketches/svg/" + jsonArray[pos])
                .attr('src', "/sketches/bitmap/" + jsonArray[pos]);

            index += 1;

            // use timeout to prevent "Image corrupt or truncated" error message in Firefox
            window.setTimeout(function() {
                $imageList.append(newImg);

                /* trigger callback after last image has been added */
                if (pos == jsonArray.length-1) {
                    callback();
                }

            }, 250);

            loadImage(jsonArray, pos+1, callback);
        }
        //endregion

        //region Privileged methods
        this.resize = resize;
        this.loadSketches = loadSketches;
        //endregion
    };

    return OpenSketchView;

})(util);

var OpenSketchPage = (function(OpenSketchView) {
    $(document).on("pageinit", "#openSketchPage", function() {
        window.openSketchView = new OpenSketchView();
    });
})(OpenSketchView);