/**
 * This file contains the logic of the PanelPopup component.
 * It is used to present and modify the metainformation about a single link (SVG rect).
 *
 * @author baltes
 * @see PanelPopup.html, PanelPopup.css
 *
 */
var PanelPopup = (function(icons, util) {

    var PanelPopup = function(args) {
        //region Private Members
        var svgCanvas,
            $popupDiv,
            element,
            isOpen,
            reopen,
            reopenElement,
            openAfterClose,
            $artifact_dom,
            $annotation_dom,
            $selectedNode_dom,
            $button_dom,
            annotation_svg,
            elementId_svg,
            annotation_old,
            artifact_id_old;

        svgCanvas = args.svgCanvas;
        $popupDiv = null;
        element = null;
        isOpen = false;
        reopen = false;
        reopenElement = null;
        openAfterClose = "";

        icons.load();
        //endregion

        //region Getter/setter for private members
        Object.defineProperties(this, {
            "element": {
                get: function() {
                    return element;
                }
            },
            "isOpen": {
                get: function() {
                    return isOpen;
                }
            },
            "openAfterClose": {
                get: function() {
                    return openAfterClose;
                },
                set: function(newValue) {
                    openAfterClose = newValue;
                }
            }
        });
        //endregion

        //region Private methods
        function create(callback) {
            $.mobile.loading('show');

            /* load css for this component */
            util.loadCSS("/com/PanelPopup/PanelPopup.css", function() {
                /* load html for this component */
                $.get("/com/PanelPopup/PanelPopup.html", function (data) {
                    var $newPopupDiv = svgCanvas.$page
                        .append($.parseHTML(data)[0])
                        .trigger('create');

                    $popupDiv = $newPopupDiv
                        .find('#panelPopup')
                        .popup();

                    $artifact_dom = $popupDiv.find('#panelArtifact');
                    $annotation_dom = $popupDiv.find('#panelAnnotation');
                    $selectedNode_dom = $popupDiv.find('#panelSelectedNode');
                    $button_dom = $popupDiv.find('#panelFollowLink');

                    util.disableTextSelection($artifact_dom);

                    /* register event handlers */
                    $(window).on('orientationchange', function() {
                        if (util.isIOSDevice()) {
                            util.unfocusInputs($popupDiv, $selectedNode_dom);
                        }
                    });

                    $(window).on('resize', function() {
                        reposition();
                    });

                    $artifact_dom
                        .on("vclick", function(event) {
                            if (util.isIOSDevice()) { // fixed bug that triggered click on canvas after clicking on the artifact text input
                                event.stopPropagation();
                                event.preventDefault();
                            }

//                            popups.navigationPopup.open({
//                                element: element
//                            });
                        })
                        .on("blur", function() {
                            util.setTextInputCursorPos($(this)[0], 0);
                        });

                    $button_dom
                        .on("vclick", function() {
                            var artifact_id = $artifact_dom.val();
                            if (artifact_id) {
                                var showBroadcast = {
                                    "id": util.generateNewID(),
                                    "message": "show",
                                    "type": "broadcast",
                                    "data": {
                                        "artifactID": artifact_id,
                                        "clients": null
                                    }
                                };

                                window.websocket.send(JSON.stringify(showBroadcast));
                            }
                        });

                    $selectedNode_dom
                        .on("blur", function() {
                            util.setTextInputCursorPos($(this)[0], 0);
                        });

                    $annotation_dom
                        .on("blur", function() {
                            saveAnnotation()
                        });

                    /* set the height of the popup to the actual visible height
                     * cp. http://jquerymobile.com/demos/1.2.0/docs/pages/popup/popup-panels.html
                     */
                    $popupDiv
                        .on('popupbeforeposition', function() {
                            reposition();
                        })
                        .on("popupafteropen", function() {
                            util.log("open_panel");

                            if (util.isIOSDevice()) {
                                util.unfocusInputs($popupDiv, $selectedNode_dom);
                            }
                            isOpen = true;
                        })
                        .on('popupafterclose', function() {
                            util.log("close_panel");

                            saveAnnotation();
                            $annotation_dom.val('');

                            if (util.isIOSDevice()) {
                                util.unfocusInputs($popupDiv, $selectedNode_dom);
                            }

                            isOpen = false;

                            if (reopen) {
                                open({
                                    element: reopenElement
                                });
                                reopenElement = null;
                                reopen = false;
                            } else {
                                switch (openAfterClose) {
                                    case "metainfo":
                                        popups.metaInfoPopup.open();
                                        popups.metaInfoPopup.openAfterClose = "panel";
                                        break;
                                    case "navigation":
                                        //popups.navigationPopup.open({
                                        //    element: element
                                        //});
                                        //popups.navigationPopup.openAfterClose = "panel";
                                        break;
                                    case "menu":
                                        popups.menuPopup.open();
                                        break;
                                    default:
                                    // do nothing
                                }

                                openAfterClose = "";
                                element = null;
                            }
                        });

                    /* call callback of create function */
                    if (callback) {
                        callback();
                    }

                    $.mobile.loading('hide');
                });

            });
        }

        function open(args) {
            var artifactPath,
                artifactType,
                pathArray,
                artifactName,
                $element,
                log;

            annotation_old = "";

            if (args && args.element) {
                element = args.element;
                $element = $(args.element);

//                if (args.clear) {
//                    $element.attr('data:artifact-path', '');
//                    $element.attr('data:artifact-type', '');
//                    util.log("artifact_cleared");
//                } else if (args.artifactPath && args.artifactType) {
//                    artifactPath = args.artifactPath;
//                    artifactType = args.artifactType;
//                    $element.attr('data:artifact-path', artifactPath);
//                    $element.attr('data:artifact-type', artifactType);
//                    log = true;
//                } else {
//                    artifactPath = $element.attr('data:artifact-path');
//                    artifactType = $element.attr('data:artifact-type'); // Case sensitivity is ignored for attributes, therefore artifact-type and not artifactType
//                }
//
//                util.updateArtifactTextInput(artifactPath, artifactType, $artifact_dom);
//
//                if (log) {
//                    util.log("artifact_changed;" + $artifact_dom.val());
//                }
//
//                annotation_svg = util.decode($element.attr('data:annotation'));
//                $annotation_dom.val(annotation_svg);

                $artifact_dom.val('');

                var marker_id =  $element.attr('id');
                // get sketch metadata
                $.get("/marker/metadata/" + marker_id, function (data) {
                    $selectedNode_dom.val(marker_id);
                    annotation_old = util.decode(data.annotation);
                    $annotation_dom.val(annotation_old);

                    if (window.changelog[marker_id]) {
                        if (window.changelog[marker_id]["annotation"]) {
                            $annotation_dom.val(window.changelog[marker_id]["annotation"]);
                        }
                    }
                });

                $.get("/marker/links/" + marker_id, function (data) {
                    if (data) {
                        artifact_id_old = data[0];
                        $artifact_dom.val(artifact_id_old);

                        if (window.code_id) {
                            $artifact_dom.val(window.code_id);

                            if (window.changelog[marker_id]) {
                                window.changelog[marker_id]["artifact-id"] = window.code_id;
                            } else {
                                window.changelog[marker_id] = {
                                    "artifact-id": window.code_id
                                }
                            }
                        }
                    }
                });

                $popupDiv.popup('open');
                isOpen = true;
            } else {
                throw new ReferenceError(APPLICATION_NAME + ": Element for panel is not defined.");
            }
        }

        function close() {
            $popupDiv.popup('close');
        }

        function closeAndReopen(newReopenElement) {
            reopen = true;
            reopenElement = newReopenElement;
            close();
        }

        function reposition() {
            $popupDiv.css("height", util.getVisibleArea().height + "px");
            $popupDiv.css("top", "-4px");
        }

        function saveAnnotation() {
//            var oldAnnotation,
//                newAnnotation;
//
//            oldAnnotation = annotation_svg;
//            newAnnotation = $annotation_dom.val();
//
//            if (newAnnotation && (newAnnotation != oldAnnotation)) {
//                $(element).attr('data:annotation', util.encode(newAnnotation));
//                annotation_svg = newAnnotation;
//
//                util.log("annotation_edited;" + util.encode(newAnnotation));
//            }

            var annotation_new,
                id;

            annotation_new = $annotation_dom.val();
            id = $selectedNode_dom.val();

            if (annotation_new != annotation_old) {
                if (!window.changelog[id]) {
                    window.changelog[id] = {
                        "annotation": annotation_new
                    }
                } else {
                    window.changelog[id]["annotation"] = annotation_new
                }
            }

        }
        //endregion

        //region Privileged methods
        this.create = create;
        this.open = open;
        this.close = close;
        this.closeAndReopen = closeAndReopen;
        //endregion
    };

    return PanelPopup;

})(icons, util);