/**
 * This component is a collection of helper functions.
 *
 * @author baltes
 *
 */
var util = (function() {

    //region Private functions
    function cleanArtifactName(artifactName) {
        // TODO : find a more general solution to clean the artifact name that not only works for Java artifacts
        /* remove modifiers from artifact name */
        return artifactName.replace(new RegExp("^(public|private|protected|static|final|native|synchronized|abstract|threadsafe|transient|\\s)*"), "").trim();
    }
    //endregion

    //region Functions
    function getVisibleArea() {
        var visibleHeight;
        var contentPaddingHeight;
        var footerHeight;
        var headerHeight;
        var viewportHeight;
        var visibleWidth;
        var contentPaddingWidth;
        var viewportWidth;
        var $content;

        window.scrollTo(0, 0);

        $content = $('[data-role="content"]');

        viewportWidth = $(window).width();
        contentPaddingWidth = parseInt($content.css("padding-left").replace("px", "")) + parseInt($content.css("padding-right").replace("px", ""));
        visibleWidth = viewportWidth - contentPaddingWidth;

        viewportHeight = $.mobile.getScreenHeight();
        headerHeight = $('[data-role="header"]').outerHeight();
        footerHeight = $('[data-role="footer"]').outerHeight();
        contentPaddingHeight = parseInt($content.css("padding-top").replace("px", "")) + parseInt($content.css("padding-bottom").replace("px", ""));
        visibleHeight = viewportHeight - headerHeight - footerHeight - contentPaddingHeight - 1; // -1 to hide scrollbar

        return {
            width: Math.max(1, visibleWidth),
            height: Math.max(1, visibleHeight)
        };
    }

    function SVG(elementName) {
        return document.createElementNS('http://www.w3.org/2000/svg', elementName);
    }

    function addEventListener(element, eventName, eventHandler, scope) {
        var scopedEventHandler;

        if (eventHandler) {
            if (scope) {
                scopedEventHandler = function (event) { eventHandler.apply(scope, [event]); };
            }
            else {
                scopedEventHandler = eventHandler;
            }

            $(element).on(eventName, scopedEventHandler);
        } else {
            throw new ReferenceError("Sketch Link: Event handler for " + eventName + " on " + $(element).attr("id") + " is " + eventHandler + ".");
        }
    }

    function disableTextSelection(element) {
        $(element)
            .attr('unselectable', 'on')
            .css('user-select', 'none')
            .on('selectstart', false);
    }

    function tokenizeStyleString(styleString) {
        var i;
        var tokenArray = styleString.replace(/ /g, '').split(/[;:]/);
        var result = [];

        for (i=0; i<tokenArray.length; i+=2) {
            result[tokenArray[i]] = tokenArray[i + 1];
        }

        return result;
    }

    function changeStyleProperty(styleString, propertyName, propertyValue) {
        var i;
        var tokenArray = styleString.replace(/ /g, '').split(/[;:]/);
        var result = "";

        for (i=0; i<tokenArray.length; i+=2) {
            if (tokenArray[i]) {
                if (tokenArray[i] == propertyName) {
                    tokenArray[i + 1] = propertyValue;
                }
                result += tokenArray[i] + ": " + tokenArray[i + 1] + "; ";
            }
        }

        return result;
    }

    function isIOSDevice() {
        if (window["IS_IOS_DEVICE"] === undefined) { // cache the result in a global variable
            window["IS_IOS_DEVICE"] = (navigator.platform == 'iPad') || (navigator.platform == 'iPhone');
        }

        return IS_IOS_DEVICE;
    }

    function getElementToScreenScaleFactor(screenWidth, screenHeight, elementWidth, elementHeight) {
        var screenAspectRatio = screenWidth / screenHeight;
        var elementAspectRatio = elementWidth / elementHeight;

        var scaleFactor = 1;

        if (screenAspectRatio <= elementAspectRatio) { // screen is narrower than element (or equal)
            scaleFactor = screenWidth / elementWidth; // use width to determine scale factor
        } else if (screenAspectRatio > elementAspectRatio) { // screen is wider than element
            scaleFactor = screenHeight / elementHeight; // use height to determine scale factor
        }

        return scaleFactor;
    }

    function tokenizePathString(pathString) {
        return pathString.split(/[/]/);
    }

    function getPathStringFromNode(node) {
        var currentNode,
            path;

        path = node.elementName;

        if (node.parent) {
            currentNode = node.parent;

            while (currentNode) {
                path = currentNode.elementName + "/" + path;
                currentNode = currentNode.parent;
            }

        }

        return path;
    }

    function validatePathArrayWithNode(node, pathArray, arrayPos) {
        var i,
            result;

        result = false;

        if (node.elementName == pathArray[arrayPos]) {
            if (node.children && (arrayPos < pathArray.length-1)) {
                for (i=0; i<node.children.length; i++) {
                    result = validatePathArrayWithNode(node.children[i], pathArray, arrayPos+1);

                    if (result) {
                        return result;
                    }
                }
            } else {
                return node;
            }
        }

        return null;
    }

    function updateArtifactTextInput(artifactPath, artifactType, $input) {
        var pathArray,
            artifactName;

        $input.val('');
        $input.removeClass('icon-none icon-project icon-module icon-entity icon-subentity icon-hand');

        if (artifactPath && artifactType) {
            pathArray = tokenizePathString(artifactPath);
            artifactName = pathArray[pathArray.length-1];
            artifactName = cleanArtifactName(artifactName);

            $input.val(artifactName);

            switch (artifactType) {
                case "project":
                    $input.addClass('icon-project');
                    break;
                case "module":
                    $input.addClass('icon-module');
                    break;
                case "entity":
                    $input.addClass('icon-entity');
                    break;
                case "subentity":
                    $input.addClass('icon-subentity');
                    break;
                default:
                    $input.addClass('icon-none');
            }
        } else {
            $input.addClass('icon-hand');
        }
    }

    function loadCSS(url, sucess) {
        if ($('style[href="' + url + '"]').exists() || $('style[data-href="' + url + '"]').exists()) {
            sucess();
            return; // stylesheet already loaded
        }

        /* load css */
        $.get(url, function(data) {
            $("<style>")
                .attr("data-href", url)
                .appendTo("head")
                .html(data);

                sucess();
        });
    }

    function isInputSelected() {
        var $activeElement = $(document.activeElement);

        if ($activeElement.is('textarea[readonly!="readonly"]')) {
            return true;
        }

        if ($activeElement.is('textarea[readonly="false"]')) {
            return true;
        }

        if ($activeElement.is('input[type="text"][readonly!="readonly"]')) {
            return true;
        }

        if ($activeElement.is('input[type="text"][readonly="false"]')) {
            return true;
        }

        return false;
    }

    function setTextInputCursorPos(input, pos) {
        input.selectionStart = 0;
        input.selectionEnd = 0;
    }

    function unfocusInputs($rootElement, $focusElement) {
        $rootElement.find('textarea').blur();
        $rootElement.find('input[type="text"]').blur();
        $focusElement.focus();
    }

    function generateNewID(prefix) {
        /* see http://stackoverflow.com/a/105074 */
        function s4() {
            return Math.floor((1 + Math.random()) * 0x10000)
                .toString(16)
                .substring(1);
        }

        var uuid4 =  s4() + s4() + '-' + s4() + '-' + s4() + '-' + s4() + '-' + s4() + s4() + s4();

        if (prefix) {
            return prefix + "-" + uuid4;
        } else {
            return uuid4;
        }
    }

    /* cp. http://stackoverflow.com/questions/487073/check-if-element-is-visible-after-scrolling */
    function isScrolledIntoView(element, scrollableContainer)
    {
        var scrollTop = $(scrollableContainer).scrollTop();
        var scrollBottom = scrollTop + $(scrollableContainer).height();

        var elementTop = $(element).offset().top;
        var elementBottom = elementTop + $(element).height();

        return ((elementBottom <= scrollBottom) && (elementTop >= scrollTop));
    }

    function enableLogging() {
         //window["LOGGING"] = true;
    }

    function disableLogging() {
        //window["LOGGING"] = false;
    }

    function log(message) {
//        if (window["LOGGING"]) {
//            var data = {
//                message: Date.now() + ";" + message
//            };
//
//            $.ajax({
//                type: "POST",
//                url: "/log/",
//                data: data,
//                dataType: "text",
//                processData: "true"
//            });
//        }
    }

    /* the newline character has to be encoded before writing to a DOM attribute since it would be replaced by a space character otherwise */
    function encode(string) {
        if (string) {
            return string.replace(/\n/g, "\\n");
        } else {
            return "";
        }
    }

    function decode(string) {
        if (string) {
            return string.replace(/\\n/g, "\n");
        } else {
            return "";
        }
    }
    //endregion

    return {
        getVisibleArea:                 getVisibleArea,
        SVG:                            SVG,
        addEventListener:               addEventListener,
        disableTextSelection:           disableTextSelection,
        tokenizeStyleString:            tokenizeStyleString,
        changeStyleProperty:            changeStyleProperty,
        isIOSDevice:                    isIOSDevice,
        getElementToScreenScaleFactor:  getElementToScreenScaleFactor,
        tokenizePathString:             tokenizePathString,
        getPathStringFromNode:          getPathStringFromNode,
        validatePathArrayWithNode:      validatePathArrayWithNode,
        updateArtifactTextInput:        updateArtifactTextInput,
        loadCSS:                        loadCSS,
        isInputSelected:                isInputSelected,
        setTextInputCursorPos:          setTextInputCursorPos,
        unfocusInputs:                  unfocusInputs,
        generateNewID:                  generateNewID,
        isScrolledIntoView:             isScrolledIntoView,
        enableLogging:                  enableLogging,
        disableLogging:                 disableLogging,
        log:                            log,
        encode:                         encode,
        decode:                         decode
    };
})();