/**
 * Logic for main page ('index.html').
 * Sets default values and registers event handlers.
 *
 * @author baltes
 * @see index.html
 *
 */
var mainpage = (function(util, SvgCanvas, MenuPopup, NewSketchPopup, OpenSketchPopup) {

    // TODO: moving on the canvas (click/tap and move)?
    // TODO: smooth scrolling and zooming (cp. http://jsfiddle.net/9zu4U/10/)?
    // TODO: test on Android
    // TODO: prevent problems with virtual keyboard in landscape mode (textareas seam to cause most problems)

    $(document).on("pageinit", "#mainpage", function() {
        var $header,
            $footer,
            svgCanvas,
            menuPopup,
            navigationPopup,
            $mainPage,
            $svgDiv;

        $.mobile.loading('show');

        /* disable hiding of header and footer with a tap and disable text selection on header and footer */
        $header = $('[data-role=header]');
        $header.fixedtoolbar({ tapToggle: false });
        util.disableTextSelection($header);

        $footer = $('[data-role=footer]');
        $footer.fixedtoolbar({ tapToggle: false });
        util.disableTextSelection($footer);

        $mainPage = $('#mainpage');
        $svgDiv = $('#svgCanvas');

        /* svg canvas */
        svgCanvas = new SvgCanvas({
            page: $mainPage,
            svgDiv: $svgDiv.get(0)
        });

        /* register event handlers for buttons */
        //$('#buttonUndo').on('vclick', svgCanvas.undo);
        //$('#buttonRedo').on('vclick', svgCanvas.redo);
        //$('#buttonZoomIn').on('vclick', svgCanvas.zoomViewboxIn);
        //$('#buttonZoomCenter').on('vclick', svgCanvas.zoomCenter);
        //$('#buttonZoomOut').on('vclick', svgCanvas.zoomViewboxOut);
        //$('#buttonMoveUp').on('vclick', svgCanvas.moveViewBoxUp);
        //$('#buttonMoveDown').on('vclick', svgCanvas.moveViewBoxDown);
        //$('#buttonMoveLeft').on('vclick', svgCanvas.moveViewBoxLeft);
        //$('#buttonMoveRight').on('vclick', svgCanvas.moveViewBoxRight);
        $('#buttonDelete').on('vclick', svgCanvas.deleteSelectedElement);
        $('#buttonMetaInfo').on('vclick', svgCanvas.openMetaInfoPopup);
        $('#buttonUpload').on('vclick', svgCanvas.uploadSvg);

        /* register event handler for keyboard shortcuts */
        $(window)
            .on('keydown', function(event) {
                /* execute event handler only if no popup is open (avoid interference) */
                if (isPopupOpen()) {
                    return;
                }

/*                switch (event.keyCode) {
                    case 46: // delete key
                        svgCanvas.deleteSelectedElement();
                        break;
                    case 37: // left arrow
                        svgCanvas.moveViewBoxLeft();
                        break;
                    case 38: // up arrow
                        svgCanvas.moveViewBoxUp();
                        break;
                    case 39: // right arrow
                        svgCanvas.moveViewBoxRight();
                        break;
                    case 40: // down arrow
                        svgCanvas.moveViewBoxDown();
                        break;
                    case 171: // plus
                        svgCanvas.zoomViewboxIn();
                        break;
                    case 173: // minus
                        svgCanvas.zoomViewboxOut();
                        break;
                    case 48: // 0
                        svgCanvas.zoomCenter();
                        break;
                    default:
                        // do nothing
                }*/
            })
            .on("click", function(event) {
                if (isPopupOpen()) {
                    return;
                }

                /*if (event.which == 2) { // middle mouse button
                    svgCanvas.zoomCenter();
                }*/
            });

        /* mouse wheel event for Chrome, Safari, etc. */
/*
        $svgDiv.get(0).addEventListener('mousewheel', function(event) {
            if (isPopupOpen()) {
                return;
            }

            if (event.wheelDelta > 0) {
                svgCanvas.zoomViewboxIn();
            } else if (event.wheelDelta < 0) {
                svgCanvas.zoomViewboxOut();
            }
        }, false);
*/

        /* mouse wheel event for Firefox */
/*        $svgDiv.get(0).addEventListener('DOMMouseScroll', function(event) {
            if (isPopupOpen()) {
                return;
            }

            if (event.detail < 0) {
                svgCanvas.zoomViewboxIn();
            } else if (event.detail > 0) {
                svgCanvas.zoomViewboxOut();
            }
        }, false);*/

        /* event handler to set title (with jQuery's "on" function, accessing event.detail was not possible) */
        document.addEventListener("setTitle", function(event) { //
            if (event.detail.text) {
                $('#headerTitle').text(event.detail.text);
                $('#headerTitleContainer').removeClass("sl-hidden");
            } else {
                $('#headerTitle').text('');
                $('#headerTitleContainer').addClass("sl-hidden");
            }
        });

        /* prevent overscroll effect on touch devices and problems with the virtual keyboard */
        $(window, document).on('scrollstart scroll', function(event) {
            event.stopPropagation();
            event.preventDefault();
            return false;
        });

        /* prevent default drag and drop in Firefox */
        $(document).bind("dragstart", function(event) {
            event.stopPropagation();
            event.preventDefault();
            return false;
        });

        /* menu popup */
        popups.menuPopup = new MenuPopup({
            $page: $mainPage,
            svgCanvas: svgCanvas
        });
        popups.menuPopup.create(function() {
            /* navigation popup */
//            popups.navigationPopup = new NavigationPopup({
//                $page: $mainPage,
//                svgCanvas: svgCanvas
//            });
//            popups.navigationPopup.create();
        });

        $('#buttonMenu').on('vclick', function(event) {
            popups.menuPopup.open({
                x: event.clientX,
                y: event.clientY
            });
        });

        // open sketch if user provided sketch ID
        if (window.sketch_id) {
            if (svgCanvas) {
                svgCanvas.initialize("/sketches/svg/"+window.sketch_id);
                window.sketch_id = "";
            }
        }

        connectWebsocket(svgCanvas);

        $.mobile.loading('hide');
    });

    function isPopupOpen() {
        return (popups.panelPopup.isOpen
            //|| popups.navigationPopup.isOpen
            || popups.metaInfoPopup.isOpen
            || popups.menuPopup.isOpen
            || popups.newSketchPopup.isOpen
            || popups.openSketchPopup.isOpen);
    }

})(util, SvgCanvas, MenuPopup, NewSketchPopup, OpenSketchPopup);


function connectWebsocket(svgCanvas) {
    window.websocket = new WebSocket(WEBSOCKET_URL);

    window.websocket.onopen = function(event) {
        console.log("Websocket open...");

        var registerRequest = {
            "id": util.generateNewID(),
            "message": "register",
            "type": "request",
            "data": {
                "clientType": "webview",
                "preferredDisplayName": "WebView",
                "protocolVersion": 1.0
            }
        };

        window.websocket.send(JSON.stringify(registerRequest));
    };

    window.websocket.onmessage = function(event) {
        console.log("Received Websocket message.");

        var jsonObject = JSON.parse(event.data);

        if (!window.sketch_id && (jsonObject.message == "show")) {
            if (svgCanvas && jsonObject.data.artifactID.startsWith("s-")) {
                svgCanvas.initialize("/sketches/svg/"+jsonObject.data.artifactID);
            }
        }
    };

    window.websocket.onclose = function(event) {
        console.log("Websocket closed.");
    }

}