__all__ = ["database", "rest_api", "webview_api"]

import sketchlink
import jinja2

from flask import Flask, g
from werkzeug.routing import BaseConverter


class RegexConverter(BaseConverter):
    """ a converter used in Flask to match URLs using regular expressions
        (see http://stackoverflow.com/questions/5870188/does-flask-support-regular-expressions-in-its-url-routing)
    """

    def __init__(self, url_map, *items):
        super(RegexConverter, self).__init__(url_map)
        self.regex = items[0]

    @staticmethod
    def attach_to(app):
        """ attaches the regex converter to a flask app
        """
        app.url_map.converters['regex'] = RegexConverter


flask_app = Flask(__name__)
flask_app.config['MAX_CONTENT_LENGTH'] = 16 * 1024 * 1024  # max. 16 MB for uploads
flask_app.jinja_loader = jinja2.ChoiceLoader([
    flask_app.jinja_loader,
    jinja2.FileSystemLoader(sketchlink.TEMPLATE_PATH), # also search in custom template path
])
RegexConverter.attach_to(flask_app)

import sketchlink.flask.rest_api
import sketchlink.flask.webview_api