import uuid

from tornado import websocket
from sketchlink.websocket.messages import Message, Request, Response, Error, Broadcast
from sketchlink.exceptions import protocol_errors


class WebSocketAPI(websocket.WebSocketHandler):
    connected_clients = []
    display_names = []

    def __init__(self, *args, **kwargs):
        self.display_name = ""
        self.identifier = "v-" + str(uuid.uuid4())
        self.ip_address = ""
        self.client_type = ""
        self.is_registered = False
        super(WebSocketAPI, self).__init__(*args, **kwargs)

    def open(self):
        self.ip_address = self.request.remote_ip
        WebSocketAPI.connected_clients.append(self)
        print("WebSocket opened: Client " + self.identifier + " connected from " + self.ip_address)

    def on_message(self, message):

        print("Received message from client " + self.identifier)

        try:
            json_message = Message.from_json(message)

            if isinstance(json_message, Request):
                print("Handling " + json_message.message_name + " request from client " + self.identifier)

                if not self.is_registered and json_message.message_name != "register":
                    raise protocol_errors.ClientNotRegisteredError(json_message)
                else:
                    response = Response.from_request(
                        json_message,
                        self,
                        WebSocketAPI.connected_clients,
                        WebSocketAPI.display_names
                    )

                    if response:
                        print("Answering to " + response.message_name + " request from client " + self.identifier)
                        self.write_message(response.to_json())

            elif isinstance(json_message, Broadcast):
                print("Handling " + json_message.message_name + " broadcast from client " + self.identifier)

                if self.is_registered:
                    broadcast, broadcast_clients = Broadcast.from_broadcast(
                        json_message,
                        self,
                        WebSocketAPI.connected_clients
                    )
                else:
                    raise protocol_errors.ClientNotRegisteredError(json_message)

                for client in broadcast_clients:
                    print("Broadcasting to client " + client.identifier + ": " + broadcast.message_name)
                    client.write_message(broadcast.to_json())

        except protocol_errors.ProtocolErrorBase as e:
            print(e.error_message)
            self.write_message(Error.from_exception(e).to_json())

    def on_close(self):
        print("WebSocket closed: Client " + self.identifier + " disconnected.")

        if self.display_name in WebSocketAPI.display_names:
            WebSocketAPI.display_names.remove(self.display_name)

        if self in WebSocketAPI.connected_clients:
            WebSocketAPI.connected_clients.remove(self)
