import os
import time
import datetime
import re
import uuid

import sketchlink
from sketchlink.database import query_db


def validate_artifact_id(artifact_id):
    return True if re.match(sketchlink.ARTIFACT_ID_RE, artifact_id) else False


def validate_client_id(client_id):
    return True if re.match(sketchlink.CLIENT_ID_RE, client_id) else False


def validate_message_id(message_id):
    return True if re.match(sketchlink.GUID_RE, message_id) else False


def exists_artifact(artifact_id):
    res = None

    if artifact_id.startswith("s"):
        query = "SELECT artifactID FROM sketch WHERE artifactID=?;"
        res = query_db(query, args=(artifact_id,), one=True)

    elif artifact_id.startswith("m"):
        query = "SELECT artifactID FROM marker WHERE artifactID=?;"
        res = query_db(query, args=(artifact_id,), one=True)

    elif artifact_id.startswith("c"):
        query = "SELECT artifactID FROM anchor WHERE artifactID=?;"
        res = query_db(query, args=(artifact_id,), one=True)

    return True if res else False


def exists_link(artifact_id_1, artifact_id_2):
    query = "SELECT artifactID1, artifactID2 FROM link WHERE (artifactID1=? AND artifactID2=?) OR " \
            "(artifactID1=? AND artifactID2=?);"

    res = query_db(query, args=(artifact_id_1, artifact_id_2, artifact_id_2, artifact_id_1), one="True")

    return True if res else False


def get_timestamp():
    return datetime.datetime.fromtimestamp(time.time()).strftime('%Y-%m-%d %H:%M:%S')


def get_id(prefix=""):
    if prefix:
        return prefix + "-" + str(uuid.uuid4())
    else:
        return str(uuid.uuid4())


def get_sketch_uri(sketch_id, filetype="svg"):
    return os.path.join(sketchlink.SKETCHES_PATH, sketch_id, sketch_id + "." + filetype)